''***********************
''*   Nyan Cat player   *
''***********************
'' Author:  Andy Schenk, Insonix.ch  Nov 2011
'' uses the PhaseMod GM Synthesizer to play the MIDI file from HubRAM.
'' You can define two sounds, or let theme choose randomly from a list.
'' The methode nyan_cat_loop can be started in a new cog and then the
'' melody is played endless in a loop.
'' Needs 1 cog for the play-loop and 1 cog for the synth.

CON
  _clkmode        = xtal1 + pll16x
  _xinfreq        = 5_000_000

  Left            = 10       '<-- Audio Out pins
  Right           = 11       '(-1 = Mono on Left)

  INSTR1          = -1       'Treble Sound (-1=random)
  INSTR2          = -1       'Bass Sound   (-1=random)
        
VAR    
  long  time, tq, ts, us
  long  ramptr, rnd
  long  stack[32]

OBJ
  synth : "pm_synth_20"
      
PUB Main
  cognew(nyan_cat_loop,@stack)


PUB nyan_cat_loop
  synth.start(Left,Right,1)                             'start synth with 10 voices (1 cog)
  rnd := cnt
  repeat
    midiplay(@midifile)                                 'play the file in a loop

PUB midiplay(pntr)  | lg, dt, b, v, st, d1, d2, ch
    ramptr := pntr
    us := clkfreq / 1_000_000                           'ticks / microsecond
    b := st := d1 := d2 := v := 0
    pread(@v,4)
    if v == "M" + "T"<<8 + "h"<<16 + "d"<<24            'Header ID
      pread(@v,4)                                       'Header Len
      v~
      pread(@v,2)                                       'Format
      if v == 0
        pread(@v,2)                                     'Tracks
        pread(@v,2)                                     'Ticks/Quarter
        tq := v>>8 + (v&255)<<8
        ts := 500_000 / tq                              'default Tempo 120 BPM
        pread(@v,4)                                     '"MTrk"
        pread(@v,4)                                     'Track length
        lg := v>>24 + (v>>16&255)<<8 + (v>>8&255)<<16
        time := cnt
        repeat
          pread(@b,1)                                   'get delta time
          lg--
          dt := b & $7F
          repeat while b>127
            pread(@b,1)
            lg--
            dt := dt<<7 + (b & $7F)
          time := time + dt*us*ts
          repeat until time-cnt < 0                     'wait deltatime
          b~
          pread(@b,1)                                   'MIDI byte
          if b > 127
            st := b                                     'new status byte
          elseif st==$FF
            st~
          ch := st & $0F
          d1~
          d2~
          case st & $F0                                 'decode MIDI Event
            $90: pread(@d1,1)                           'Note 
                 pread(@d2,1)                           'Velocity
                 lg -= 3
                 if d2>0
                   synth.noteOn(d1,ch,d2)               'Note On
                 else
                   synth.noteOff(d1,ch)                 'Note Off if Vel=0
            $80: pread(@d1,1)                           'Note Off 
                 pread(@d2,1)                           'Velocity
                 lg -= 3
                 synth.noteOff(d1,ch)
            $A0: pread(@d1,1)                           'Poly AfterTouch 
                 pread(@d2,1)                           '(not supported)
                 lg -= 3
            $C0: pread(@d1,1)                           'Prg Change 
                 lg -= 2
                 if ch==0
                   d1 := INSTR1
                 if ch==1
                   d1 := INSTR2
                 if d1 < 0
                    d1 := sndlist[(rnd?) & 15]
                 synth.prgChange(d1,ch)
            $D0: pread(@d1,1)                           'Mono AfterTouch 
                 lg -= 2                                '(not supported)
            $B0: pread(@d1,1)                           'Controller (nr) 
                 pread(@d2,1)                           'value
                 lg -= 3
                 if d1==7
                   synth.volContr(d2,ch)                '7=Vol
            $E0: pread(@d1,1)                           'Pitch Bender 
                 pread(@d2,1)                           '(not supported!)
                 lg -= 3
            $F0: pread(@d1,1)                           'Meta Event
                 pread(@b,1)                            'Len
                 lg -= 2
                 v~
                 repeat while b>0                       'read meta data
                   pread(@d2,1)
                   v := v<<8 + d2
                   b--
                   lg--
                   if d1==5 and v&$80 > 0
                     quit
                 if d1==81                              'Tempo
                   ts := v / tq
        until lg < 12
'        until lg < 1
'    synth.allOff

PUB pread(rptr,size)                                    ''Read RAM data
  repeat size
    byte[rptr++] := byte[ramptr++]
          
DAT
sndlist   byte  1,6,9,25,29,30,38,45,48,56,60,62,76,80,81,87   '16 random sounds

midifile  file  "NYAN_CAT.MID"
  